
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>


enum {
	// _xinfreq = 20_000_000,
	_xtlfreq = 20_000_000,
	_clkfreq = 100_000_000,
	DOWNLOAD_BAUD = 230_400,
	DEBUG_BAUD = DOWNLOAD_BAUD,

};



uint32_t  pinwait( int Temp_pin, uint32_t mode )
{
    uint32_t  i, sample, diff1 = 0, diff2 = 0;

    _rdpin( Temp_pin );
    _wrpin( Temp_pin, mode );
    while( !_pinr( Temp_pin ) );
    _rdpin( Temp_pin );

    i = 3;
    do {
        while( !_pinr( Temp_pin ) );    // wait for next decimation
        sample = _rdpin( Temp_pin ) << 5;    // 27-bit to 32-bit translation for clean rotary summing
        sample -= diff1;
        diff1 += sample;
        sample -= diff2;
        diff2 += sample;
    } while(--i);

    return sample >> 10; // sinc3 x 7 (128 period) + 5 (translation) - 10 = 16-bit samples
}


//enum  PINBIAS = P_ADC_1X | P_ADC
enum  {PINBIAS = P_ADC_1X | P_ADC | P_OE | P_LOW_150K};


void  tsr_loop( int Temp_pin )
{
    uint32_t  i, value, vio, gio, timer;
    float  temperature, ratio;

    _pinstart( Temp_pin, PINBIAS, 0b10_0000, 128 ); // "Sinc3 Filtering" mode, 128 ticks per decimation, rated 14 ENOB
    _waitms( 10 );    // warm up the ADC hardware circuits
    printf( "      pin P%d\n", Temp_pin );

    for( i = 0; i<8; i++ ) {
        timer = _cnt();
        vio = pinwait( Temp_pin, P_ADC_VIO | P_ADC );
        gio = pinwait( Temp_pin, P_ADC_GIO | P_ADC );
        value = pinwait( Temp_pin, PINBIAS );    // must be last in order, no idea why
        timer = _cnt() - timer;
        printf( "duration = %d  ", timer );

        ratio = (float)(vio - gio) / (float)(value - gio) - 1.0;    // Rt connected to GIO, R0 connected to VIO
//        ratio = (float)(vio - gio) / (float)(vio - value) - 1.0;    // Rt connected to VIO, R0 connected to GIO
        temperature = 1.0 / (log(ratio) / 3380.0 + 1.0 / 298.15) - 273.15;    // Thermistor Beta for 10 kR at 25 C

        printf( "gio = %d  vio = %d  value = %d  Temperature = %f  Ratio = %f  rln = %f\n",
                gio, vio, value, temperature, ratio, log(ratio) );
        _waitms( 100 );
    }

    _pinclear( Temp_pin );    // turn off the ADC
}



void main(void)
{

    _waitms(200);
    printf( "\n   clkfreq = %d   clkmode = 0x%x\n", _clockfreq(), _clockmode() );

    for(int i = 0; i<62; i++)
        tsr_loop(i);
}
